/*	$OpenBSD: grep.h,v 1.14 2007/09/02 15:19:32 deraadt Exp $	*/
/*	$FreeBSD$	*/
/*-
 * Copyright (c) 1999 James Howard and Dag-Erling Codan Smrgrav
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <bzlib.h>
#include <regex.h>
#include <stdio.h>
#include <zlib.h>

#ifdef WITH_PCRE
#include <pcre.h>
#endif

#ifdef WITHOUT_NLS
#define getstr(n)	 errstr[n]
#else
#include <nl_types.h>

extern nl_catd		 catalog;
#define getstr(n)	 catgets(catalog, 1, n, errstr[n])
#endif

extern char		*errstr[];

#define VERSION		"2.5.1-FreeBSD"

#define GREP_FIXED	0
#define GREP_BASIC	1
#define GREP_EXTENDED	2
#define GREP_PERL	3

#define BINFILE_BIN	0
#define BINFILE_SKIP	1
#define BINFILE_TEXT	2

#define FILE_STDIO	0
#define FILE_GZIP	1
#define FILE_BZIP	2

#define DIR_GREP	0
#define DIR_SKIP	1
#define DIR_RECURSE	2

#define DEV_GREP	0
#define DEV_SKIP	1

#define LINK_GREP	0
#define LINK_EXPLICIT	1
#define LINK_SKIP	2

#define MAX_LINE_MATCHES	32

struct file {
	int		 binary;
	FILE		*f;
	struct mmfile	*mmf;
	gzFile		*gzf;
	BZFILE		*bzf;
};

struct str {
	size_t		 len;
	int		 line_no;
	off_t		 off;
	char		*file;
	char		*dat;
};

/* Flags passed to regcomp() and regexec() */
extern int	 cflags, eflags;

/* Command line flags */
extern int	 Eflag, Fflag, Gflag, Hflag, Lflag,
		 bflag, cflag, hflag, iflag, lflag, mflag, nflag, oflag,
		 qflag, sflag, vflag, wflag, xflag;
extern int	 nullflag, exclflag, inclflag;
extern unsigned long long Aflag, Bflag, mcount;
extern char	*color, *label;
extern int	 grepbehave, binbehave, filebehave, devbehave, dirbehave, linkbehave;

extern int	 first, prev, matchall, patterns, epatterns, tail, notfound;
extern char    **pattern, **epattern;
extern regex_t	*r_pattern, *er_pattern;

#ifdef WITH_PCRE
extern pcre	**perl_pattern;
#endif


/* For regex errors  */
#define RE_ERROR_BUF	512
extern char	 re_error[RE_ERROR_BUF + 1];	/* Seems big enough */

/* util.c */
int	 procfile(char *fn);
int	 grep_tree(char **argv);
void	*grep_malloc(size_t size);
void	*grep_calloc(size_t nmemb, size_t size);
void	*grep_realloc(void *ptr, size_t size);
void	 printline(struct str *line, int sep, regmatch_t *matches, int m);

/* queue.c */
void	 initqueue(void);
void	 enqueue(struct str *x);
void	 printqueue(void);
void	 clearqueue(void);

/* file.c */
char		*binbufptr;

void		 grep_close(struct file *f);
struct file	*grep_stdin_open(void);
struct file	*grep_open(char *path);
int		 grep_feof(struct file *f);
int		 grep_fgetc(struct file *f);
char		*grep_fgetln(struct file *f, size_t *len);
