/*-
 * Copyright (c) 1999 James Howard and Dag-Erling Codan Smrgrav
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * A really poor man's queue.  It does only what it has to and gets out of
 * Dodge.
 */

#include <sys/cdefs.h>

#ifndef lint
#if 0
__RCSID("$OpenBSD: queue.c,v 1.5 2006/02/09 09:54:47 otto Exp $");
#else
__FBSDID("$FreeBSD$");
#endif
#endif /* not lint */

#include <sys/param.h>

#include <stdlib.h>
#include <string.h>

#include "grep.h"

struct queue {
	struct queue	*next;
	struct str	 data;
};

static struct queue	*q_head, *q_tail;
static int		 count;

static struct queue	*dequeue(void);

void
initqueue(void)
{
	q_head = q_tail = NULL;
}

static void
free_item(struct queue *item)
{

	free(item);
}

void
enqueue(struct str *x)
{
	struct queue	*item;

	item = grep_malloc(sizeof *item + x->len);
	item->data.len = x->len;
	item->data.line_no = x->line_no;
	item->data.off = x->off;
	item->data.dat = (char *)item + sizeof *item;
	memcpy(item->data.dat, x->dat, x->len);
	item->data.file = x->file;
	item->next = NULL;

	if (!q_head) {
		q_head = q_tail = item;
	} else {
		q_tail->next = item;
		q_tail = item;
	}

	if (++count > Bflag)
		free_item(dequeue());
}

static struct queue *
dequeue(void)
{
	struct queue	*item;

	if (q_head == NULL)
		return (NULL);

	--count;
	item = q_head;
	q_head = item->next;
	if (q_head == NULL)
		q_tail = NULL;
	return (item);
}

void
printqueue(void)
{
	struct queue	*item;

	while ((item = dequeue()) != NULL) {
		printline(&item->data, '-', (regmatch_t *)NULL, 0);
		free_item(item);
	}
}

void
clearqueue(void)
{
	struct queue	*item;

	while ((item = dequeue()) != NULL)
		free_item(item);
}
